<?php

namespace Drupal\forms_steps;

use Drupal\Core\Config\Entity\ConfigEntityInterface;

/**
 * Interface FormsStepsInterface.
 *
 * @package Drupal\forms_steps
 */
interface FormsStepsInterface extends ConfigEntityInterface {

  /**
   * Adds a step to the forms_steps.
   *
   * @param string $step_id
   *   The step's ID.
   * @param string $label
   *   The step's label.
   * @param string $nodeType
   *   The step's node_type.
   * @param string $formMode
   *   The step's form_mode.
   * @param string $url
   *   The step's URL.
   * @param string $stepMode
   *   The step's mode (edit|add).
   *
   * @return \Drupal\forms_steps\FormsStepsInterface
   *   The forms_steps entity.
   */
  public function addStep($step_id, $label, $nodeType, $formMode, $url, $stepMode);

  /**
   * Determines if the forms_steps has a step with the provided ID.
   *
   * @param string $step_id
   *   The step's ID.
   *
   * @return bool
   *   TRUE if the forms_steps has a step with the provided ID, FALSE if not.
   */
  public function hasStep($step_id);

  /**
   * Gets step objects for the provided step IDs.
   *
   * @param string[] $step_ids
   *   A list of step IDs to get. If NULL then all steps will be returned.
   *
   * @return \Drupal\forms_steps\StepInterface[]
   *   An array of forms_steps steps.
   *
   * @throws \InvalidArgumentException
   *   Thrown if $step_ids contains a step ID that does not exist.
   */
  public function getSteps(array $step_ids = NULL);

  /**
   * Gets a forms_steps step.
   *
   * @param string $step_id
   *   The step's ID.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps step.
   *
   * @throws \InvalidArgumentException
   *   Thrown if $step_id does not exist.
   */
  public function getStep($step_id);

  /**
   * Sets a step's label.
   *
   * @param string $step_id
   *   The step ID to set the label for.
   * @param string $label
   *   The step's label.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepLabel($step_id, $label);

  /**
   * Sets a step's weight value.
   *
   * @param string $step_id
   *   The step ID to set the weight for.
   * @param int $weight
   *   The step's weight.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepWeight($step_id, $weight);

  /**
   * Sets a step's Node type.
   *
   * @param string $step_id
   *   The step ID to set the node_type for.
   * @param int $nodeType
   *   The step's node type.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepNodeType($step_id, $nodeType);

  /**
   * Sets a step's form mode value.
   *
   * @param string $step_id
   *   The step ID to set the form mode for.
   * @param int $formMode
   *   The step's form mode.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepFormMode($step_id, $formMode);

  /**
   * Sets a step's URL value.
   *
   * @param string $step_id
   *   The step ID to set the URL for.
   * @param int $url
   *   The step's URL.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepUrl($step_id, $url);

  /**
   * Sets a step's mode.
   *
   * @param string $step_id
   *   The step ID to set the mode for.
   * @param int $mode
   *   The step's mode.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepMode($step_id, $mode);

  /**
   * Sets a step's submit label.
   *
   * @param string $step_id
   *   The step ID to set the submit label for.
   * @param string $label
   *   The step's submit label.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepSubmitLabel($step_id, $label);

  /**
   * Sets a step's cancel label.
   *
   * @param string $step_id
   *   The step ID to set the cancel label for.
   * @param string $label
   *   The step's cancel label.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepCancelLabel($step_id, $label);

  /**
   * Sets a step's cancel route.
   *
   * @param string $step_id
   *   The step ID to set the route for.
   * @param string $route
   *   The step's cancel route.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepCancelRoute($step_id, $route);

  /**
   * Sets a step's cancel step.
   *
   * @param string $step_id
   *   The step ID to set the cancel step for.
   * @param \Drupal\forms_steps\Step $step
   *   The step's cancel step.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepCancelStep($step_id, Step $step);

  /**
   * Sets a step's cancel step mode.
   *
   * @param string $step_id
   *   The step ID to set the cancel step mode for.
   * @param string $mode
   *   The step's cancel step mode.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   */
  public function setStepCancelStepMode($step_id, $mode);

  /**
   * Deletes a step from the forms_steps.
   *
   * @param string $step_id
   *   The step ID to delete.
   *
   * @return \Drupal\forms_steps\StepInterface
   *   The forms_steps entity.
   *
   * @throws \InvalidArgumentException
   *   Thrown if $step_id does not exist.
   */
  public function deleteStep($step_id);

  /**
   * Returns the next step to $step.
   *
   * @param \Drupal\forms_steps\Step $step
   *    The current Step.
   *
   * @return \Drupal\forms_steps\Step|null
   *    Returns the next Step or null if no next step found.
   */
  public function getNextStep(Step $step);

  /**
   * Set the label of the delete button of the step
   *
   * @param int $step_id
   *    Step id.
   * @param mixed $label
   *    Label to set.
   *
   * @return \Drupal\forms_steps\Entity\FormsSteps
   *    The forms steps.
   */
  public function setStepDeleteLabel($step_id, $label);

  /**
   * Set the delete state (hidden or shown) of the step.
   *
   * @param int $step_id
   *    Step id.
   * @param bool $state
   *    State to set.
   *
   * @return \Drupal\forms_steps\Entity\FormsSteps
   *    The forms steps.
   */
  public function setStepDeleteState($step_id, $state);

}
