<?php

namespace Drupal\Tests\forms_steps\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\Role;

/**
 * Tests for the Forms Steps module.
 *
 * @group forms_steps
 * @runInSeparateProcess
 */
class FormsStepsTest extends BrowserTestBase {

  /** Data used to build the test. */
  private $data;

  /**
   * Modules to install.
   *
   * @var array
   */
  public static $modules = [
    'node',
    'field',
    'field_ui',
    'form_mode_manager',
    'forms_steps',
  ];

  /**
   * A simple user.
   *
   * @var object
   */
  private $user;

  /**
   * Perform initial setup tasks that run before every test method.
   */
  public function setUp() {
    parent::setUp();

    $permissions = [
      'administer forms_steps',
      'administer content types',
      'administer nodes',
      'administer forms_steps',
      'administer display modes',
      'administer node fields',
      'administer node display',
      'administer node form display',
    ];

    $this->data = [
      'form_display_modes' => [
        1 => [
          'label' => 'Test Form Mode',
          'id' => 'test_form_mode',
        ],
      ],
      'forms_steps' => [
        'label' => 'Test Form Step',
        'id' => 'test_form_step',
        'description' => 'Test Form Step description',
        'steps' => [
          1 => [
            'label' => 'Add Test Step',
            'id' => 'add_test_step',
            'target_form_mode' => 'node.test_form_mode',
            'target_node_type' => 'article',
            'url' => '/my_test_form/step_1',
            'step_mode' => 'add',
          ],
          2 => [
            'label' => 'Edit Test Step',
            'id' => 'edit_test_step',
            'target_form_mode' => 'node.test_form_mode',
            'target_node_type' => 'article',
            'url' => '/my_test_form/step_2',
            'step_mode' => 'edit',
          ],
        ],
      ],
    ];

    $this->checkPermissions($permissions);
    $this->user = $this->drupalCreateUser($permissions);

    // Login.
    $this->drupalLogin($this->user);
  }

  /**
   * Tests the creation and use of a form step.
   */
  public function testFormsSteps() {
    // Creation of all form display modes.
    foreach ($this->data['form_display_modes'] as $form_display_mode) {
      // Access form mode add page.
      $this->drupalGet('admin/structure/display-modes/form/add/node');
      $this->assertSession()->statusCodeEquals(200);
      // Add a form mode.
      $this->drupalPostForm(NULL, [
        'label' => $form_display_mode['label'],
        'id' => $form_display_mode['id'],
      ], t('Save'));

      $this->assertSession()->pageTextContains(
        'Saved the ' . $form_display_mode['label'] . ' form mode.'
      );

      Role::load($this->user->getRoles()[1])
        ->grantPermission('use node.' . $form_display_mode['id'] . ' form mode')
        ->save();
    }

    // Create Article content type.
    $this->createContentType([
      'type' => 'article',
    ]);

    Role::load($this->user->getRoles()[1])
      ->grantPermission("administer nodes")
      ->save();

    // Access article's form display page.
    $this->drupalGet('admin/structure/types/manage/article/form-display');
    $this->assertSession()->statusCodeEquals(200);

    // Activate Test Form Modes as a custom display mode.
    foreach ($this->data['form_display_modes'] as $form_display_mode) {
      $this->drupalPostForm(NULL, [
        "display_modes_custom[${form_display_mode['id']}]" => $form_display_mode['id'],
      ], t('Save'));

      $this->assertSession()->pageTextContains(
        "The ${form_display_mode['label']} mode now uses custom display settings."
      );
    }

    // Configure the visible fields.
    $this->drupalGet("admin/structure/types/manage/article/form-display/${form_display_mode['id']}");
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalPostForm(NULL, [
      'fields[title][region]' => 'content',
      'fields[body][region]' => 'hidden',
      'fields[status][region]' => 'hidden',
      'fields[uid][region]' => 'hidden',
      'fields[created][region]' => 'hidden',
      'fields[promote][region]' => 'hidden',
      'fields[sticky][region]' => 'hidden',
    ], t('Save'));

    // Access forms steps add page.
    $this->drupalGet('admin/config/workflow/forms_steps/add');
    $this->assertSession()->statusCodeEquals(200);
    // Test the creation of a form step.
    $this->drupalPostForm(NULL, [
      'label' => $this->data['forms_steps']['label'],
      'id' => $this->data['forms_steps']['id'],
      'description' => $this->data['forms_steps']['description'],
    ], t('Save'));
    $this->assertSession()->pageTextContains(
      'Forms Steps ' . $this->data['forms_steps']['label'] . ' has been created.'
    );

    // Perform steps creation.
    foreach ($this->data['forms_steps']['steps'] as $step) {
      // Access step add page of the form step.
      $this->drupalGet('admin/config/workflow/forms_steps/test_form_step/add_state');
      $this->assertSession()->statusCodeEquals(200);

      // Test the creation of an add step.
      $this->drupalPostForm(NULL, [
        'label' => $step['label'],
        'id' => $step['id'],
        'target_form_mode' => $step['target_form_mode'],
        'target_node_type' => $step['target_node_type'],
        'url' => $step['url'],
        'step_mode' => $step['step_mode'],
      ], t('Save'));

      $this->assertSession()->pageTextContains(
        'Created ' . $step['label'] . ' step.'
      );
    }

    // Perform basic form mode testing.
    // Access Create Article as Test Form Mode page.
    $this->drupalGet("node/add/article/${form_display_mode['id']}");
    $this->assertSession()->statusCodeEquals(200);
    // Test if the body field is editable (it should not be)
    $this->assertSession()->fieldNotExists(
      'body[0][value]'
    );

    // Test the creation of an article node as Test Form Mode.
    $this->drupalPostForm(NULL, [
      'title[0][value]' => "${form_display_mode['label']} Test Node",
    ], t('Save'));
    $this->assertSession()->pageTextContains(
      "Article ${form_display_mode['label']} Test Node has been created."
    );

    // Test the flow/
    // Access the step 1.
    $this->drupalGet($this->data['forms_steps']['steps'][1]['url']);

    // Check status code.
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()
      ->pageTextContains($this->data['forms_steps']['steps'][1]['label']);

    $value = 'This is a Test Titre content';
    $this->drupalPostForm(NULL, [
      'title[0][value]' => $value,
    ], t('Save'));

    $this->assertSession()
      ->pageTextContains($this->data['forms_steps']['steps'][2]['label']);
    $this->assertContains($this->data['forms_steps']['steps'][2]['url'], $this->getUrl());
    $this->assertSession()->pageTextContains($value);
  }

}
